﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Net;
using System.IO;
using System.Drawing;

using FluentFTP;

namespace BlendoFTPUploader
{


    public partial class FileList : Form
    {
        Form1 initialForm;
        FtpClient client;
        string localpath, remotepath;

        public FileList(FtpClient _client, string _localpath, string _remotepath, Form1 _initialForm)
        {
            client = _client;
            localpath = _localpath;
            remotepath = _remotepath;
            initialForm = _initialForm;

            this.FormClosing += new System.Windows.Forms.FormClosingEventHandler(ManualClose);
            

            InitializeComponent();

            listview_FileList.ItemSelectionChanged += yourListView_ItemSelectionChanged;
        }

        private void yourListView_ItemSelectionChanged(object sender, ListViewItemSelectionChangedEventArgs e)
        {
            if (e.IsSelected)
                e.Item.Selected = false;
        }


        protected override void OnShown(EventArgs e)
        {
            base.OnShown(e);



            SetDebugLog(string.Format("FTP connection = {0}!", client.IsConnected));
            


            if (!client.DirectoryExists(remotepath))
            {
                MessageBox.Show(string.Format("Unable to find FTP folder: {0}\n\nDisconnecting...", remotepath), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                CloseWindow(true);
                return;
            }


            //get listing of local files.
            DirectoryInfo localdir = new DirectoryInfo(localpath);
            FileInfo[] localfiles = localdir.GetFiles("*.*", SearchOption.AllDirectories);

            if (localfiles.Length <= 0)
            {
                MessageBox.Show(string.Format("Local folder is empty!\n{0}\n\nDisconnecting...", localpath), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                CloseWindow(true);
                return;
            }

            SetDebugLog(string.Format("Found {0} local files.", localfiles.Length));


            //get listing of remote files.
            SetDebugLog("Retrieving remote files, please wait......");
            FtpListItem[] remoteFiles = client.GetListing(remotepath, FtpListOption.ForceList | FtpListOption.Recursive);

            SetDebugLog(string.Format("Found {0} remote files.", remoteFiles.Length));

            //make dictionary of remote files.
            Dictionary<string, FtpListItem> remoteDictionary = new Dictionary<string, FtpListItem>(remoteFiles.Length);
            for (int i = 0; i < remoteFiles.Length; i++)
            {
                string adjustedRemoteString = remoteFiles[i].FullName;
                adjustedRemoteString = adjustedRemoteString.Remove(0, remotepath.Length + 1);

                if (adjustedRemoteString.StartsWith("/") || adjustedRemoteString.StartsWith("\\"))
                {
                    adjustedRemoteString = adjustedRemoteString.Remove(0, 1); //remove leading slash.
                }

                if (remoteFiles[i].Type != FtpFileSystemObjectType.File)
                    continue;

                remoteDictionary.Add(adjustedRemoteString, remoteFiles[i]);
            }


            //now........ do the comparisons.
            SetDebugLog("Comparing files, please wait......");

            for (int i = 0; i < localfiles.Length; i++)
            {
                string adjustedLocalString = localfiles[i].FullName;
                adjustedLocalString = adjustedLocalString.Remove(0, localpath.Length + 1);
                adjustedLocalString = adjustedLocalString.Replace('\\', '/');

                FtpListItem foundItem;
                if (!remoteDictionary.TryGetValue(adjustedLocalString, out foundItem))
                {
                    //file does NOT exist on the remote server. Add entry!
                    AddListEntry(adjustedLocalString, "Add");
                    this.Update();
                }
                else if (foundItem != null)
                {

                    //compare file sizes and also date.

                    if ((localfiles[i].Length != foundItem.Size)
                        && (DateTime.Compare(localfiles[i].LastWriteTime, foundItem.Modified) > 0))
                    {
                        SetDebugLog(string.Format( "Local: {0}  {1} bytes / remote: {2} bytes", adjustedLocalString, localfiles[i].Length, foundItem.Size));

                        AddListEntry(adjustedLocalString, "Overwrite");
                    }

                }
            }


            SetDebugLog("Done comparing files!");

            
            if (listview_FileList.Items.Count <= 0)
            {
                MessageBox.Show("No changes found!\n\nDisconnecting...", string.Empty, MessageBoxButtons.OK, MessageBoxIcon.Information);
                CloseWindow(true);
                return;
            }

            

            button_upload.Enabled = true;
            button1_disconnect.Enabled = true;

            button_upload.Focus();
            button_upload.Select();
        }
        

        private void AddListEntry(string localfile,  string operation)
        {
            string destination = Path.Combine(remotepath, localfile);
            destination = destination.Replace('\\', '/');

            ListViewItem newItem = new ListViewItem();
            newItem.Text = localfile;
            newItem.SubItems.Add(destination);
            newItem.SubItems.Add(operation);
            listview_FileList.Items.Add(newItem);
            //this.Update();
            this.Refresh();
        }

        private void ManualClose(object sender, FormClosingEventArgs e)
        {
            CloseWindow(false);
        }

        private void button1_disconnect_Click(object sender, EventArgs e)
        {
            CloseWindow(true);
        }

        private void CloseWindow(bool forceClose)
        {
            Enabled = false;

            if (Enabled)
                return;

            client.Disconnect();

            if (forceClose)
                this.Close();

            initialForm.SetAllControlsEnabled(true);
        }

        private void button_upload_Click(object sender, EventArgs e)
        {
            //click the upload button.
            SetDebugLog(string.Format("Starting upload of {0} files...", listview_FileList.Items.Count));

            button_upload.Enabled = false;
            button1_disconnect.Enabled = false;
            this.Update();

            List<string> failedFiles = new List<string>();

            for (int i = 0; i < listview_FileList.Items.Count; i++)
            {
                string localFilepath = Path.Combine(localpath, listview_FileList.Items[i].Text);

                string remoteFilepath = Path.Combine(remotepath, listview_FileList.Items[i].SubItems[0].Text);

                if (client.UploadFile(localFilepath, remoteFilepath, FtpExists.Overwrite, true, FtpVerify.None))
                {
                    //success!
                    listview_FileList.Items[i].SubItems[1].ForeColor = Color.LimeGreen;
                    SetDebugLog(string.Format("Uploaded: {0}", listview_FileList.Items[i].Text));
                }
                else
                {
                    //fail :(
                    listview_FileList.Items[i].SubItems[1].ForeColor = Color.Red;
                    SetDebugLog(string.Format("FAILED: {0}", listview_FileList.Items[i].Text));
                }

                this.Update();
                this.Refresh();                    
            }

            if (failedFiles.Count <= 0)
            {
                SetDebugLog(string.Format("DONE! All {0} files successfully uploaded.", listview_FileList.Items.Count));
            }
            else
            {
                SetDebugLog(string.Format("DONE, but with ERRORS. Failed to upload {0} files:", failedFiles.Count));

                for (int i = 0; i < failedFiles.Count; i++)
                {
                    SetDebugLog("    " + failedFiles[i]);
                }
            }

            button1_disconnect.Enabled = true;
        }

        private void SetDebugLog(string text)
        {
            listbox_output.Items.Add(text);

            int nItems = (int)(listbox_output.Height / listbox_output.ItemHeight);
            listbox_output.TopIndex = listbox_output.Items.Count - nItems;


            this.Update();
            this.Refresh();

            this.Update();
            this.Refresh();

        }
    }
}
